import 'dart:convert';

import 'package:flutter_cache/flutter_cache.dart' as cache;
import 'package:http/http.dart' as http;

import 'package:jiosaavn_wrapper/constants/strings.dart';
import 'package:jiosaavn_wrapper/factories/song_factory.dart';
import 'package:jiosaavn_wrapper/modals/playlist.dart';
import 'package:jiosaavn_wrapper/modals/search_result.dart';
import 'package:jiosaavn_wrapper/modals/song.dart';

class JioSaavnWrapper {
  static final JioSaavnWrapper instance = JioSaavnWrapper._internal();

  JioSaavnWrapper._internal();

  Future<Playlist> fetchTopSongs({int? expiredAt = 86400}) async {
    try {
      var map = await cache.remember(
        'topSongs',
        () async {
          var response = await http.get(
            Uri.parse(topSongsURL),
            headers: {"Access-Control-Allow-Origin": "*"},
          );
          return Map<String, dynamic>.from(jsonDecode(response.body));
        },
        expiredAt,
      );
      return await PluginFactory.createPlayList(map);
    } catch (e) {
      throw e;
    }
  }

  Future<Song> fetchSongDetails({required String songId}) async {
    try {
      String songUrl = songDetailURL + songId;
      var map = await cache.remember(
        'songDetails_$songId',
        () async => Map<String, dynamic>.from(json.decode(await http
            .get(Uri.parse(songUrl), headers: {
          "Accept": "application/json"
        }).then((value) => value.body.split("-->")[1]))[songId]),
      );
      return await PluginFactory.createSong(map);
    } catch (e) {
      throw e;
    }
  }

  Future<SearchResult> fetchSearchResults({required String searchQuery}) async {
    try {
      var map = await cache.remember('searchResultsof$searchQuery', () async {
        var response = await http.get(Uri.parse(searchUrl(searchQuery)),
            headers: {"Accept": "application/json"});
        var resEdited = (response.body).split("-->");
        return Map<String, dynamic>.from(json.decode(resEdited[1]));
      });
      return PluginFactory.createSearchResult(map);
    } catch (e) {
      throw e;
    }
  }

  Future<TopSearchResult> fetchTrendingSearchResult() async {
    try {
      var map = await cache.remember('trendingSearchResult', () async {
        var response = await http.get(
          Uri.parse(trendingSearchURL),
          headers: {"Accept": "application/json"},
        );
        return List.from(json.decode(response.body));
      });
      return PluginFactory.createTrendingSearchResult(map);
    } catch (e) {
      throw e;
    }
  }
}
